@@ -7,10 +7,10 @@ use warnings FATAL => 'all';
 ###########################################################################
 
 { package Set::Relation; # role
-    use version 0.74; our $VERSION = qv('0.7.0');
+    use version 0.74; our $VERSION = qv('0.8.0');
     # Note: This given version applies to all of this file's packages.
 
-    use Moose::Role 0.69;
+    use Moose::Role 0.72;
 
     use namespace::clean -except => 'meta';
 
@@ -89,7 +89,7 @@ use warnings FATAL => 'all';
 
 { package Set::Relation::Mutable; # role
 
-    use Moose::Role 0.69;
+    use Moose::Role 0.72;
 
     use namespace::clean -except => 'meta';
 
@@ -122,18 +122,20 @@ Relation data type for Perl
 
 =head1 VERSION
 
-This document describes Set::Relation version 0.7.0 for Perl 5.
+This document describes Set::Relation version 0.8.0 for Perl 5.
 
 =head1 SYNOPSIS
 
-    use Set::Relation::V1;
+    use Set::Relation::V2;
 
-    my $r1 = Set::Relation::V1->new( [ [ 'x', 'y' ], [
+    sub relation { return Set::Relation::V2->new( @_ ); }
+
+    my $r1 = relation( [ [ 'x', 'y' ], [
         [ 4, 7 ],
         [ 3, 2 ],
     ] ] );
 
-    my $r2 = Set::Relation::V1->new( [
+    my $r2 = relation( [
         { 'y' => 5, 'z' => 6 },
         { 'y' => 2, 'z' => 1 },
         { 'y' => 2, 'z' => 4 },
@@ -230,11 +232,7 @@ Despite initial design efforts to help Set::Relation's execution (CPU, RAM,
 etc) performance, this module is still assumed to be very un-optimized for
 its conceptually low level task of data crunching.  It generally applies
 the same internal representation and algorithms regardless of the actual
-structure or meaning of the data, and regardless of the amount of data.  It
-generically applies certain up-front costs in the form of data hashing that
-should both speed up later operations and simplify the implementation code
-of most operations, but any actual performance benefit depends a lot on
-actual use, and it may even have a net loss of execution performance.
+structure or meaning of the data, and regardless of the amount of data.
 
 This module is still assumed to be considerably, perhaps an order or three
 of magnitude, slower than a hand-rolled task-specific solution.  If your
@@ -441,7 +439,7 @@ being reference types, that compare on their memory addresses.  Value types
 can't be mutated while the containers that the reference values point to
 can be mutated.  If you want some other class' object treated as a value
 type, make it overload stringification I<(or an alternate/additional
-convention can be devised like Set::Relation's own 'which' convention)>. 
+convention can be devised like Set::Relation's own 'which' convention)>.
 If you want a Hash-ref/tuple or Set::Relation object to be treated as a
 reference type, then pass it around using another layer of reference
 indirection, such as by adding a scalar-ref up front.
@@ -450,6 +448,56 @@ I<Note: Were this Perl 6, we would basically just use the standard C<WHICH>
 method or C<===> comparison operator to determine identity; but Perl 5
 doesn't have that so we do the aforementioned instead.>
 
+=head2 Matters of Correctness
+
+Set::Relation explicitly supports implementations that want to give users
+the option of trading away specified guarantees of correctness for their
+user-visible behaviour in exchange for better performance (by letting the
+implementation be lazier), when an approximation is good enough for the
+users' purposes.  But Set::Relation requires that implementations fully
+guarantee correctness (insomuch as they reasonably have the power to do so)
+for their user-visible behaviour by default, and only trade this for
+laziness when users explicitly request the trade.  Of course,
+implementations can always be internally lazy as they see fit where it
+doesn't impact their externally observable semantics, so the trading
+feature under discussion here is only about behaviour that users can see.
+
+For example, some Set::Relation routines have the optional boolean
+parameter C<$allow_dup_tuples> which, when explicitly given a true
+argument, tells the implementation it may save itself the work of comparing
+tuples with each other to test their uniqueness, and so multiple instances
+of the same tuple value may be treated in user-visible ways as being
+distinct when they logically aren't.  For example, if users construct a
+Set::Relation-doing object using a list of tuples that contains duplicates,
+then by default that object's C<cardinality> method will only result in a
+count of distinct tuples, and that object's C<body> method will only result
+in a list containing distinct tuples, because a relation is logically a set
+of tuples not a multiset; but if a true C<$allow_dup_tuples> argument is
+given to either method, then the result may include duplicates as if the
+class were implementing a multiset rather than a set.  Similarly, internal
+duplicates can arise out of none in many other situations than object
+construction, such as from a relational union or projection.  Note that
+because Perl closures in general are not pure and so might have different
+results or side-effects from repeated invocations with the same arguments,
+the C<$allow_dup_tuples> parameter is also present on methods like
+C<restriction> and C<extension>.
+
+It is important to realize that even when users explicitly waive a need for
+correctness, the implementation may choose to ignore them and give the
+fully correct answers anyway.  Trade-off options should not be construed as
+having a guarantee to not do the normal behaviour, and in particular a true
+C<$allow_dup_tuples> argument will I<not> guarantee consistent logical
+multiset behaviour.
+
+It is an design decision feature of Set::Relation that flags like
+C<$allow_dup_tuples> are only available directly on the methods where they
+would take effect just on specific invocations, and are not available as
+object attributes.  This is to prevent action at a distance, where one spot
+in code requesting less accuracy doesn't also cause other spots in code
+using the same objects to also get less accuracy.  And so you may have to
+use said flag explicitly multiple times to get continuous such laziness
+through the lifecycle of an object or chain of derived objects.
+
 =head2 Matters of Mutability
 
 This module defines both the Set::Relation role and the
@@ -483,7 +531,7 @@ Note:  See also the L</Appropriate Uses For Set::Relation> section above.
 
 The matters of performance can differ significantly depending on the
 implementation class, so see each of their corresponding documentation
-sections for details: L<Set::Relation::V1>.
+sections for details: L<Set::Relation::V1>, L<Set::Relation::V2>.
 
 =head1 INTERFACE
 
@@ -530,17 +578,19 @@ L<HDMD_Perl5_Tiny|Muldis::D::Dialect::HDMD_Perl5_Tiny> for a C<QRelation>
 are just ordinary Perl values and not HDMD_Perl5_Tiny value literal nodes.
 Examples are:
 
+    sub relation { return $sr_class_name->new( @_ ); }
+
     # Zero attrs + zero tuples.
-    my $r1 = Set::Relation->new( [] );
+    my $r1 = relation( [] );
 
     # 3 attrs + zero tuples.
-    my $r2 = Set::Relation->new( [ 'x', 'y', 'z' ] );
+    my $r2 = relation( [ 'x', 'y', 'z' ] );
 
     # Zero attrs + 1 tuple
-    my $r3 = Set::Relation->new( [ {} ] );
+    my $r3 = relation( [ {} ] );
 
     # Named attributes format: 3 attrs + 1 tuple.
-    my $r4 = Set::Relation->new( [
+    my $r4 = relation( [
         {
             'login_name' => 'hartmark',
             'login_pass' => 'letmein',
@@ -549,7 +599,7 @@ Examples are:
     ] );
 
     # Ordered attributes format: 2 attrs + 1 tuple.
-    my $r5 = Set::Relation->new( [ [ 'name', 'age' ], [
+    my $r5 = relation( [ [ 'name', 'age' ], [
         [ 'Michelle', 17 ],
     ] ] );
 
@@ -557,13 +607,13 @@ However, Set::Relation also supports a few additional, trivial formats for
 C<$members>, as illustrated here:
 
     # The default value of a Set::Relation has zero attrs + zero tuples.
-    my $r6 = Set::Relation->new();
+    my $r6 = relation();
 
     # One way to clone a relation object.
-    my $r7 = Set::Relation->new( $r5 );
+    my $r7 = relation( $r5 );
 
     # Abbreviated way to specify 1 attr + zero tuples.
-    my $r8 = Set::Relation->new( 'value' );
+    my $r8 = relation( 'value' );
 
 =head1 Accessor Methods
 
@@ -572,12 +622,13 @@ attributes, essentially the reverse process of an object constructor.
 
 =head2 export_for_new
 
-C<method export_for_new of Hash ($self: Bool|Array $want_ord_attrs?)>
+C<method export_for_new of Hash ($self: Bool|Array $want_ord_attrs?, Bool
+$allow_dup_tuples?)>
 
 This method results in a Perl Hash value whose Hash keys and values you can
 give as argument names and values to C<new> such that the latter would
-result in a clone of this method's invocant, as if you had used C<clone>.
-In other words, C<export_for_new> is the inverse function to C<new>.  If
+result in a clone of this method's invocant.  In other words,
+C<export_for_new> is the inverse function to C<new>.  If
 this method's C<$want_ord_attrs> argument is missing or false, then the
 exported attributes and tuples are in named attributes format; if that
 argument is true, they are in ordered attributes format.  If
@@ -586,7 +637,9 @@ attribute names matching those of the invocant, and so it is specifying
 what order the attributes should be in the result; otherwise if
 C<$want_ord_attrs> is the Perl string value C<1>, then the result will have
 its attributes ordered alphabetically by attribute name (see the C<heading>
-method docs for why that is the case).
+method docs for why that is the case).  If this method's
+C<$allow_dup_tuples> argument is true, then the result may have
+duplicate tuples; otherwise (the default), all result tuples are unique.
 
 =head2 which
 
@@ -608,7 +661,8 @@ C<which> (code inside said classes will do conversions as needed).
 
 =head2 members
 
-C<method members of Array ($self: Bool|Array $want_ord_attrs?)>
+C<method members of Array ($self: Bool|Array $want_ord_attrs?, Bool
+$allow_dup_tuples?)>
 
 This method results in a Perl Array value as per the 'members' element of
 the Hash that C<export_for_new> would result in with the same invocant and
@@ -630,28 +684,34 @@ method is currently an alias for the C<attr_names> functional method.
 
 =head2 body
 
-C<method body of Array ($self: Bool|Array $want_ord_attrs?)>
+C<method body of Array ($self: Bool|Array $want_ord_attrs?, Bool
+$allow_dup_tuples?)>
 
 This method results in a Perl Array value whose elements are the tuples of
 the invocant.  Each tuple is either a Perl Hash or a Perl Array depending
-on the value of the C<$want_ord_attrs>, like with the C<members> method.
+on the value of the C<$want_ord_attrs>, like with the C<members> method;
+similarly C<$allow_dup_tuples> affects the result as with C<members>.
 
 =head2 slice
 
-C<method slice of Array ($self: Array|Str $attrs, Bool $want_ord_attrs?)>
+C<method slice of Array ($self: Array|Str $attr_names, Bool
+$want_ord_attrs?, Bool $allow_dup_tuples?)>
 
 This method is like C<body> except that the result has just a subset of the
-attributes of the invocant, those named by C<$attrs>.  Unlike using
+attributes of the invocant, those named by C<$attr_names>.  Unlike using
 C<projection> followed by C<body> to do this, any duplicate subtuples are
 retained in the result of C<slice>.  Each result subtuple is either a Perl
 Hash or a Perl Array depending on the value of the C<$want_ord_attrs>, like
 with C<body>, except that C<$want_ord_attrs> may only be a Bool here; when
 that argument is true, the exported attributes are in the same order as
-specified in C<$attrs>.
+specified in C<$attr_names>.  If this method's C<$allow_dup_tuples>
+argument is false (the default), then the result is guaranteed to have the
+same number of elements as the cardinality of C<$self>; otherwise, the
+result may have more elements.
 
 =head2 attr
 
-C<method attr of Array ($self: Str $name)>
+C<method attr of Array ($self: Str $name, Bool $allow_dup_tuples?)>
 
 This method is like C<slice> except that the result has exactly one of the
 invocant's attributes, the one named by C<$name>, and each result element
@@ -688,7 +748,7 @@ zero (that is, it has zero attributes), and false otherwise.
 
 =head2 has_attrs
 
-C<method has_attrs of Bool ($topic: Array|Str $attrs)>
+C<method has_attrs of Bool ($topic: Array|Str $attr_names)>
 
 This functional method results in true iff, for every one of the attribute
 names specified by its argument, its invocant has an attribute with that
@@ -705,10 +765,14 @@ accessor method.
 
 =head2 cardinality
 
-C<method cardinality of UInt ($topic:)>
+C<method cardinality of UInt ($topic: Bool $allow_dup_tuples?)>
 
 This functional method results in the cardinality of its invocant (that is,
-the count of tuples its body has).
+the count of tuples its body has).  If this method's C<$allow_dup_tuples>
+argument is false (the default), then the result is guaranteed to only
+count the number of distinct tuples of C<$topic>; otherwise, the result may
+be higher, unless the invocant is empty, in which case the result is still
+exactly zero.
 
 =head2 is_empty
 
@@ -770,19 +834,19 @@ that aren't being renamed.
 
 =head2 projection
 
-C<method projection of Set::Relation ($topic: Array|Str $attrs)>
+C<method projection of Set::Relation ($topic: Array|Str $attr_names)>
 
 This functional method results in the relational projection of its
 C<$topic> invocant that has just the subset of attributes of C<$topic>
-which are named in its C<$attrs> argument.  As a trivial case, this
-method's result is C<$topic> if C<$attrs> lists all attributes of
-C<$topic>; or, it is a nullary relation if C<$attrs> is empty.  This method
-will fail if C<$attrs> specifies any attribute names that C<$topic> doesn't
-have.
+which are named in its C<$attr_names> argument.  As a trivial case, this
+method's result is C<$topic> if C<$attr_names> lists all attributes of
+C<$topic>; or, it is a nullary relation if C<$attr_names> is empty.  This
+method will fail if C<$attr_names> specifies any attribute names that
+C<$topic> doesn't have.
 
 =head2 cmpl_projection
 
-C<method cmpl_projection of Set::Relation ($topic: Array|Str $attrs)>
+C<method cmpl_projection of Set::Relation ($topic: Array|Str $attr_names)>
 
 This functional method is the same as C<projection> but that it results in
 the complementary subset of attributes of its invocant when given the same
@@ -911,7 +975,8 @@ at all levels, of some specified part).
 
 =head2 restriction
 
-C<method restriction of Set::Relation ($topic: Code $func)>
+C<method restriction of Set::Relation ($topic: Code $func, Bool
+$allow_dup_tuples?)>
 
 This functional method results in the relational restriction of its
 C<$topic> invocant as determined by applying the Bool-resulting
@@ -926,29 +991,35 @@ relation with the same heading.  Note that this operation is also
 legitimately known as I<where>.  See also the C<semijoin> method, which is
 a simpler-syntax alternative for C<restriction> in its typical usage where
 restrictions are composed simply of anded or ored tests for attribute value
-equality.
+equality.  If this method's C<$allow_dup_tuples> argument is false (the
+default), then C<$func> is guaranteed to be invoked just once per distinct
+tuple of C<$topic>; otherwise it might be multiple invoked.
 
 =head2 restriction_and_cmpl
 
-C<method restriction_and_cmpl of Array ($topic: Code $func)>
+C<method restriction_and_cmpl of Array ($topic: Code $func, Bool
+$allow_dup_tuples?)>
 
 This functional method performs a 2-way partitioning of all the tuples of
-C<$topic> and results in a 2-element Perl Array whose 2 element values are
-each Set::Relation objects; the first and second elements are what
-C<restriction> and C<cmpl_restriction>, respectively, would result in when
-having the same invocant and argument.
+C<$topic> and results in a 2-element Perl Array whose element values are
+each Set::Relation objects that have the same heading as C<$topic> and
+complementary subsets of its tuples; the first and second elements are
+what C<restriction> and C<cmpl_restriction>, respectively, would result in
+when having the same invocant and arguments.
 
 =head2 cmpl_restriction
 
-C<method cmpl_restriction of Set::Relation ($topic: Code $func)>
+C<method cmpl_restriction of Set::Relation ($topic: Code $func, Bool
+$allow_dup_tuples?)>
 
 This functional method is the same as C<restriction> but that it results in
 the complementary subset of tuples of C<$topic> when given the same
-arguments.  See also the C<semidifference> method.
+invocant and arguments.  See also the C<semidifference> method.
 
 =head2 extension
 
-C<method extension of Set::Relation ($topic: Array|Str $attrs, Code $func)>
+C<method extension of Set::Relation ($topic: Array|Str $attr_names, Code
+$func, Bool $allow_dup_tuples?)>
 
 This functional method results in the relational extension of its C<topic>
 invocant as determined by applying the tuple/Hash-resulting zero-parameter
@@ -957,19 +1028,21 @@ relation has a heading that is a superset of that of C<$topic>, and its
 body contains the same number of tuples, with all attribute values of
 C<$topic> retained, and possibly extra present, determined as follows; for
 each C<$topic> tuple, the subroutine given in C<$func> results in a second
-tuple when the first tuple is its C<$_> topic; the
-first and second tuples must have no attribute names in common, and the
-result tuple is derived by joining (cross-product) the tuples together.  As
-a trivial case, if C<$func> is defined to unconditionally result in the
-degree-zero tuple, then this method results simply in C<$topic>.  Now,
-C<extension> requires the extra C<$attrs> argument to prevent ambiguity in
-the general case where C<$topic> might have zero tuples, because in that
-situation, C<$func> would never be invoked, and the names of the attributes
-to add to C<$topic> are not known (we don't generally assume that
-C<extension> can reverse-engineer C<$func> to see what attributes it would
-have resulted in).  This method will fail if C<$topic> has at least 1
-tuple and the result of C<$func> does not have matching attribute names to
-those named by C<$attrs>.
+tuple when the first tuple is its C<$_> topic; the first and second tuples
+must have no attribute names in common, and the result tuple is derived by
+joining (cross-product) the tuples together.  As a trivial case, if
+C<$func> is defined to unconditionally result in the degree-zero tuple,
+then this method results simply in C<$topic>.  Now, C<extension> requires
+the extra C<$attr_names> argument to prevent ambiguity in the general case
+where C<$topic> might have zero tuples, because in that situation, C<$func>
+would never be invoked, and the names of the attributes to add to C<$topic>
+are not known (we don't generally assume that C<extension> can
+reverse-engineer C<$func> to see what attributes it would have resulted
+in).  This method will fail if C<$topic> has at least 1 tuple and the
+result of C<$func> does not have matching attribute names to those named by
+C<$attr_names>.  If this method's C<$allow_dup_tuples> argument is false
+(the default), then C<$func> is guaranteed to be invoked just once per
+distinct tuple of C<$topic>; otherwise it might be multiple invoked.
 
 =head2 static_extension
 
@@ -983,8 +1056,8 @@ are given in the C<$attrs> argument.
 
 =head2 map
 
-C<method map of Set::Relation ($topic: Array|Str $result_attrs, Code
-$func)>
+C<method map of Set::Relation ($topic: Array|Str $result_attr_names, Code
+$func, Bool $allow_dup_tuples?)>
 
 This functional method provides a convenient one-place generalization of
 per-tuple transformations that otherwise might require the chaining of up
@@ -1003,18 +1076,21 @@ C<$func> is defined to unconditionally result in the same tuple as its own
 C<$topic> argument, then this method results simply in C<$topic>; or, if
 C<$func> is defined to have a static result, then this method's result
 will have just 0..1 tuples.  Now, C<map> requires the extra
-C<$result_attrs> argument to prevent ambiguity in the general case where
-C<$topic> might have zero tuples, because in that situation, C<$func> would
-never be invoked, and the names of the attributes of the result are not
-known (we don't generally assume that C<map> can reverse-engineer C<$func>
-to see what attributes it would have resulted in).  This method will fail
-if C<$topic> has at least 1 tuple and the result of C<$func> does not have
-matching attribute names to those named by C<$result_attrs>.
+C<$result_attr_names> argument to prevent ambiguity in the general case
+where C<$topic> might have zero tuples, because in that situation, C<$func>
+would never be invoked, and the names of the attributes of the result are
+not known (we don't generally assume that C<map> can reverse-engineer
+C<$func> to see what attributes it would have resulted in).  This method
+will fail if C<$topic> has at least 1 tuple and the result of C<$func> does
+not have matching attribute names to those named by C<$result_attr_names>.
+If this method's C<$allow_dup_tuples> argument is false (the default), then
+C<$func> is guaranteed to be invoked just once per distinct tuple of
+C<$topic>; otherwise it might be multiple invoked.
 
 =head2 summary
 
 C<method summary of Set::Relation ($topic: Array|Str $group_per, Array|Str
-$result_attrs, Code $summ_func)>
+$result_attr_names, Code $summ_func, Bool $allow_dup_tuples?)>
 
 This functional method provides a convenient context for using aggregate
 functions to derive a per-group summary relation, which is its result, from
@@ -1038,14 +1114,17 @@ C<$topic> relation at once (except by chance of it resolving to 1 group);
 you should instead invoke your summarize-all C<$summ_func> directly, or
 inline it, rather than by way of C<summary>, especially if you want a
 single-tuple result on an empty C<$topic> (which C<summary>) won't do.
-Now, C<summary> requires the extra C<$result_attrs> argument to prevent
-ambiguity in the general case where C<$topic> might have zero tuples,
-because in that situation, C<$summ_func> would never be invoked, and the
-names of the attributes of the result are not known (we don't generally
-assume that C<summary> can reverse-engineer C<$summ_func> to see what
-attributes it would have resulted in).  This method will fail if C<$topic>
-has at least 1 tuple and the result of C<$summ_func> does not have matching
-attribute names to those named by C<$result_attrs>.
+Now, C<summary> requires the extra C<$result_attr_names> argument to
+prevent ambiguity in the general case where C<$topic> might have zero
+tuples, because in that situation, C<$summ_func> would never be invoked,
+and the names of the attributes of the result are not known (we don't
+generally assume that C<summary> can reverse-engineer C<$summ_func> to see
+what attributes it would have resulted in).  This method will fail if
+C<$topic> has at least 1 tuple and the result of C<$summ_func> does not
+have matching attribute names to those named by C<$result_attr_names>.  If
+this method's C<$allow_dup_tuples> argument is false (the default), then
+C<$summ_func> is guaranteed to be invoked just once per distinct post-group
+tuple; otherwise it might be multiple invoked.
 
 =head1 Multiple Input Relation Functional Methods
 
@@ -1169,9 +1248,10 @@ I<antijoin> or I<anti-semijoin>.
 C<method semijoin_and_diff of Array ($source: Set::Relation $filter)>
 
 This functional method performs a 2-way partitioning of all the tuples of
-C<$source> and results in a 2-element Perl Array whose 2 element values are
-each Set::Relation objects; the first and second elements are what
-C<semijoin> and C<semidifference>, respectively, would result in when
+C<$source> and results in a 2-element Perl Array whose element values are
+each Set::Relation objects that have the same heading as C<$source> and
+complementary subsets of its tuples; the first and second elements are
+what C<semijoin> and C<semidifference>, respectively, would result in when
 having the same invocant and argument.
 
 =head2 semijoin
@@ -1319,8 +1399,8 @@ to supporting substitutions.
 
 =head2 substitution
 
-C<method substitution of Set::Relation ($topic: Array|Str $attrs, Code
-$func)>
+C<method substitution of Set::Relation ($topic: Array|Str $attr_names, Code
+$func, Bool $allow_dup_tuples?)>
 
 This functional method is similar to C<extension> except that it
 substitutes values of existing relation attributes rather than adding new
@@ -1340,10 +1420,13 @@ C<substitution> could conceivably be implemented such that each result from
 C<$func> is allowed to specify replacement values for different subsets of
 C<$topic> attributes; however, to improve the method's predictability and
 ease of implementation over disparate foundations, C<substitution> requires
-the extra C<$attrs> argument so that users can specify a consistent subset
-that C<$func> will update (possibly to itself).  This method will fail if
-C<$topic> has at least 1 tuple and the result of C<$func> does not have
-matching attribute names to those named by C<$attrs>.
+the extra C<$attr_names> argument so that users can specify a consistent
+subset that C<$func> will update (possibly to itself).  This method will
+fail if C<$topic> has at least 1 tuple and the result of C<$func> does not
+have matching attribute names to those named by C<$attr_names>.  If this
+method's C<$allow_dup_tuples> argument is false (the default), then
+C<$func> is guaranteed to be invoked just once per distinct tuple of
+C<$topic>; otherwise it might be multiple invoked.
 
 =head2 static_substitution
 
@@ -1357,22 +1440,22 @@ attributes; the new attribute values are given in the C<$attrs> argument.
 =head2 subst_in_restr
 
 C<method subst_in_restr of Set::Relation ($topic: Code $restr_func,
-Array|Str $subst_attrs, Code $subst_func)>
+Array|Str $subst_attr_names, Code $subst_func, Bool $allow_dup_tuples?)>
 
 This functional method is like C<substitution> except that it only
 transforms a subset of the tuples of C<$topic> rather than all of them.  It
 is a short-hand for first separating the tuples of C<$topic> into 2 groups
 where those passed by a relational restriction (defined by C<$restr_func>)
-are then transformed (defined by C<$subst_attrs> and C<$subst_func>), then
-the result of the substitution is unioned with the un-transformed group.
-See also the C<subst_in_semijoin> method, which is a simpler-syntax
+are then transformed (defined by C<$subst_attr_names> and C<$subst_func>),
+then the result of the substitution is unioned with the un-transformed
+group.  See also the C<subst_in_semijoin> method, which is a simpler-syntax
 alternative for C<subst_in_restr> in its typical usage where restrictions
 are composed simply of anded or ored tests for attribute value equality.
 
 =head2 static_subst_in_restr
 
 C<method static_subst_in_restr of Set::Relation ($topic: Code $restr_func,
-Hash $subst)>
+Hash $subst, Bool $allow_dup_tuples?)>
 
 This functional method is to C<subst_in_restr> what C<static_substitution>
 is to C<substitution>.  See also the C<static_subst_in_semijoin> method.
@@ -1380,7 +1463,7 @@ is to C<substitution>.  See also the C<static_subst_in_semijoin> method.
 =head2 subst_in_semijoin
 
 C<method subst_in_semijoin of Set::Relation ($topic: Set::Relation $restr,
-Array|Str $subst_attrs, Code $subst_func)>
+Array|Str $subst_attr_names, Code $subst_func, Bool $allow_dup_tuples?)>
 
 This functional method is like C<subst_in_restr> except that the subset of
 the tuples of C<$topic> to be transformed is determined by those matched by
@@ -1438,13 +1521,16 @@ those missing attribute values.
 =head2 outer_join_with_exten
 
 C<method outer_join_with_exten of Set::Relation ($primary: Set::Relation
-$secondary, Code $exten_func)>
+$secondary, Code $exten_func, Bool $allow_dup_tuples?)>
 
 This functional method is the same as C<outer_join_with_static_exten> but
 that the result tuples from non-matches are the result of performing a
 relational extension on the un-matched C<$primary> tuples such that each
 said result tuple is determined by applying the Perl subroutine given in
-C<$exten_func> to each said C<$primary> tuple.
+C<$exten_func> to each said C<$primary> tuple.  If this method's
+C<$allow_dup_tuples> argument is false (the default), then C<$exten_func>
+is guaranteed to be invoked just once per un-matched tuple of
+C<$secondary>; otherwise it might be multiple invoked.
 
 =head1 THE Set::Relation::Mutable ROLE
 
@@ -1540,7 +1626,7 @@ L<version-ver(0.74..*)|version>.
 
 It also requires these Perl 5 packages that are on CPAN:
 L<namespace::clean-ver(0.09..*)|namespace::clean>,
-L<Moose::Role-ver(0.69..*)|Moose::Role>.
+L<Moose::Role-ver(0.72..*)|Moose::Role>.
 
 =head1 INCOMPATIBILITIES
 
@@ -1554,6 +1640,10 @@ mainly based on.  The Muldis D language in turn has as a primary influence
 the work of Christopher J. Date and Hugh Darwen whose home website is
 L<http://www.thethirdmanifesto.com/>.
 
+These Perl 5 packages that are in the current distribution are classes that
+implement the Set::Relation roles: L<Set::Relation::V1>,
+L<Set::Relation::V2>.
+
 These other Perl 6 packages: L<Muldis::Rosetta>, L<Set>.
 
 These other Perl 5 packages: L<Muldis::Rosetta>, L<Set::Object>,
